' This script runs a command line program and copies its standard output
' to the script's standard output. You can do other things with
' the output: write it to a file, parse it to extract information,
' and so on. The purpose of this sample is to show the structure
' you need to use to deal with a program that generates an arbitrary
' amount of output.
'
' The trick is to periodically read whatever data is available from the exec
' pipeline while the program is running, and then read whatever is left after
' the program terminates. Normally, the inner ReadLine loop will do all the work.
' ReadLine "blocks" -- halts the script -- if there is no data to read, until
' the program writes something or terminates.
'
' I have seen situations where AtEndOfStream returns True while the command is still
' running, hence an outer loop is needed to keep trying as long as
' necessary. (I wish I could remember the circumstances under which this occurs!)
'
' If you want, you can eliminate the code that counts the number of lines of text
' (the code that involves variable nlines).
'
' You could also add code to read from StdErr. However, you can't read from both StdOut
' and StdErr while the program is running. Here's what could happen: if the script called ReadLine
' to read from, say, StdErr and there was nothing to read, the script would block -- stop
' -- until the program wrote something to StdErr or terminated. Meanwhile, the program could be
' writing a lot of stuff to StdOut. If it writes enough to fill the storage space Windows set aside
' for the pipeline between the programs, the application would block too, waiting for the
' script to read stuff out of the pipeline. Now, both programs would be stopped, each
' waiting for the other to do something. You'd have no choice but to type Ctrl-C to kill the script.
'
' If you do want to read from both StdOut and StdErr, you have to hope that the program
' will write very little to one, and have the first loop read from the other. Usually,
' StdErr gets very little output, so you'll read from StdOut in the first loop and hope
' that the application doesn't block writing to StdErr. Then after the exec.Terminate
' command, you can add another loop to drain any data sitting in StdErr.
'
' -----------------------------------------------------------------------------------

    dim shell, cmdline, nlines, failed, exec, str, status

    cmdline = "tracert www.microsoft.com"		' construct the command line. If you want to run a
							' built-in command like "dir", you must precede it
							' with "cmd /c ", e.g. cmdline = "cmd /c dir c:\"

    nlines   = 0					' = number of lines of output collected
    failed   = False					' = flag set True if the program reported an error

    set shell = CreateObject("WScript.Shell")		' create the shell utility object (do this just once)

    on error resume next				' don't halt script on error

    set exec = shell.Exec(cmdline)			' create a Shell.Exec object & start running the application

    if err then						' if command failed to start, print an error message
	wscript.echo "Could not execute command:", cmdline
	wscript.quit 					' terminate script
    end if

    on error goto 0					' back to normal behavior, halt script on error

    do while exec.Status = 0				' loop as long as the application is still running
	WScript.Sleep 1000				' wait one second to give the program a chance to run
							' read text stored in the pipe, as much as is available
	do while (exec.status = 0) and not exec.StdOut.AtEndOfStream
	    str    = exec.StdOut.ReadLine		' read and count one line
	    nlines = nlines+1
	
 	    wscript.echo str				' copy text to output (you can do whatever you want with it)
	loop
    loop						' if program is still active, go back and read some more

    status = exec.ExitCode				' get program's exit status
    if status <> 0 then failed = True			' if the program returned a non-zero value, it usually means there was a problem

    exec.Terminate					' tell Windows that we're done with the application

    do while not exec.StdOut.AtEndOfStream		' now, drain any remaining text from the StdOut pipe
	str    = exec.StdOut.ReadLine
	nlines = nlines+1

	wscript.echo str				' copy text to output (again, you can do whatever you want with it)
    loop

    set exec = Nothing					' release the exec object

    if failed then					' report failure
	wscript.echo "(Program failed, status =", status & ")"
    end if
